from django.test import TestCase
from django.apps import apps
from django.core.urlresolvers import reverse

from wagtail.wagtailcore.rich_text import RichText
from wagtail.wagtailcore.models import Collection, Page
from wagtail.wagtailimages.tests.utils import Image, get_test_image_file

from wagtail_feeds.models import RSSFeedsSettings

from .models import HomePage, BlogPage, BlogStreamPage


class WagtailFeedTests(TestCase):
    def setUp(self):
        ContentType = apps.get_model('contenttypes.ContentType')
        Site = apps.get_model('wagtailcore.Site')

        # Delete the default homepage
        Page.objects.get(id=2).delete()

        # Create content type for homepage model
        homepage_content_type, created = ContentType.objects.get_or_create(
            model='HomePage', app_label='tests')

        # Create a new homepage
        homepage = HomePage.objects.create(
            title="Homepage",
            slug='home',
            content_type=homepage_content_type,
            path='00010001',
            depth=1,
            url_path="/home-page",
        )

        # Create a site with the new homepage set as the root
        site = Site.objects.create(
            hostname='localhost', root_page=homepage, is_default_site=True)

        RSSFeedsSettings.objects.create(site=site, feed_app_label='tests',
                                        feed_model_name='BlogStreamPage',
                                        feed_title='Test Feed',
                                        feed_link="https://example.com",
                                        feed_description="Test Description",
                                        feed_item_description_field="intro",
                                        feed_item_content_field="body",
                                        feed_image_in_content=True)

        # Create collection for image
        img_collection  = Collection.objects.create(name="test", depth=1)

        # Create an image
        image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
            collection=img_collection,
        )

        blogpage_content_type, created = ContentType.objects.get_or_create(
            model='BlogPage', app_label='tests')

        # Create Blog Page
        BlogPage.objects.create(
            title="BlogPage",
            intro="Welcome to Blog",
            body="This is the body of blog",
            date="2016-06-30",
            slug='blog-post',
            url_path="/home-page/blog-post/",
            content_type=blogpage_content_type,
            feed_image=image,
            path='000100010002',
            depth=2,
        )

        stream_blogpage_content_type, created = ContentType.objects.get_or_create(
            model='BlogStreamPage', app_label='tests')

        # Create Stream Field Blog Page

        stream_page = BlogStreamPage.objects.create(
            title="BlogStreamPage",
            intro="Welcome to Blog Stream Page",
            body=[
            ('heading', 'foo'),
            ('paragraph', RichText(
                '<p>Rich text</p><div style="padding-bottom: 56.25%;"' +
                ' class="responsive-object"> <iframe width="480" height="270"' +
                ' src="https://www.youtube.com/embed/mSffkWuCkgQ?feature=oembed"' +
                ' frameborder="0" allowfullscreen=""></iframe>' +
                '<img alt="wagtail.jpg" height="500"' +
                ' src="/media/images/wagtail.original.jpg" width="1300">' +
                '</div>'))],
            date="2016-08-30",
            slug='blog-stream-post',
            url_path="/home-page/blog-stream-post/",
            content_type=stream_blogpage_content_type,
            feed_image=image,
            path='000100010003',
            depth=3,
        )

    def test_settings_values(self):
        feed_settings = RSSFeedsSettings.objects.first()
        self.assertEqual(feed_settings.feed_app_label, 'tests')
        self.assertEqual(feed_settings.feed_model_name, 'BlogStreamPage')
        self.assertEqual(feed_settings.feed_title, 'Test Feed')
        self.assertEqual(feed_settings.feed_item_description_field, 'intro')
        self.assertEqual(feed_settings.feed_item_content_field, 'body')

    def test_blog_values(self):
        blog = BlogPage.objects.first()
        self.assertEqual(blog.intro, 'Welcome to Blog')
        self.assertEqual(blog.body, 'This is the body of blog')

    def test_stream_blogpage_values(self):
        stream_blogpage = BlogStreamPage.objects.first()
        body = stream_blogpage.body
        self.assertEqual(body[0].value, 'foo')
        self.assertIsInstance(body[1].value, RichText)
        self.assertHTMLEqual(
            body[1].value.source,
            '<p>Rich text</p><div style="padding-bottom: 56.25%;"' +
            ' class="responsive-object"> <iframe width="480" height="270"' +
            ' src="https://www.youtube.com/embed/mSffkWuCkgQ?feature=oembed"' +
            ' frameborder="0" allowfullscreen=""></iframe>' +
            '<img alt="wagtail.jpg" height="500"' +
            ' src="/media/images/wagtail.original.jpg" width="1300">' +
            '</div>'
        )

    #def test_rss_basic_generation(self):
    #    response = self.client.get(reverse('basic_feed'))
    #    self.assertEqual(response.status_code, 200)

    #def test_rss_extended_generation(self):
    #    response = self.client.get(reverse('extended_feed'))
    #    self.assertEqual(response.status_code, 200)

    #def test_rss_extended_generation_without_feed_image(self):
    #    stream_page = BlogStreamPage.objects.first()
    #    stream_page.feed_image = None
    #    stream_page.save()
    #    response = self.client.get(reverse('extended_feed'))
    #    self.assertEqual(response.status_code, 200)



import django

SECRET_KEY = 'fake-key'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sites',

    'wagtail.contrib.settings',
    'wagtail.contrib.modeladmin',
    'wagtail.contrib.table_block',
    'wagtail.wagtailforms',
    'wagtail.wagtailsearch',
    'wagtail.wagtailembeds',
    'wagtail.wagtailimages',
    'wagtail.wagtailsites',
    'wagtail.wagtailusers',
    'wagtail.wagtailsnippets',
    'wagtail.wagtaildocs',
    'wagtail.wagtailadmin',
    'wagtail.wagtailcore',

    'taggit',

    'wagtail_feeds',
    'tests',
)

DATABASES = {
    'default': {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "wagtail"
    }
}

if django.VERSION >= (1, 10):
    MIDDLEWARE = (
        'django.middleware.common.CommonMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',

        'wagtail.wagtailcore.middleware.SiteMiddleware',
    )
else:
    MIDDLEWARE_CLASSES = (
        'django.middleware.common.CommonMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',

        'wagtail.wagtailcore.middleware.SiteMiddleware',
    )


ROOT_URLCONF = 'tests.urls'
WAGTAIL_SITE_NAME = "Test Site",
SITE_ID = 1



# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2016-09-29 11:30
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_feeds', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='rssfeedssettings',
            name='feed_image_in_content',
            field=models.BooleanField(default=True, help_text=b'Add feed image to content encoded field'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-10 10:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import wagtail.wagtailcore.blocks
import wagtail.wagtailcore.fields
import wagtail.wagtailimages.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0032_add_bulk_delete_page_permission'),
        ('wagtailimages', '0018_remove_rendition_filter'),
    ]

    operations = [
        migrations.CreateModel(
            name='BlogPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('intro', wagtail.wagtailcore.fields.RichTextField()),
                ('body', wagtail.wagtailcore.fields.RichTextField()),
                ('date', models.DateField(verbose_name='Post date')),
                ('feed_image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='BlogStreamPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('intro', wagtail.wagtailcore.fields.RichTextField()),
                ('body', wagtail.wagtailcore.fields.StreamField([('heading', wagtail.wagtailcore.blocks.CharBlock(classname='full title')), ('paragraph', wagtail.wagtailcore.blocks.RichTextBlock()), ('image', wagtail.wagtailimages.blocks.ImageChooserBlock())])),
                ('date', models.DateField(verbose_name='Post date')),
                ('feed_image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='HomePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



from django.db import models

from wagtail.contrib.settings.models import BaseSetting, register_setting

# Create your models here.


@register_setting
class RSSFeedsSettings(BaseSetting):
    feed_app_label = models.CharField(
        max_length=255, help_text='blog App whose Feed is to be generated',
        null=True, blank=True)
    feed_model_name = models.CharField(
        max_length=255, help_text='Model to be used for feed generation',
        null=True, blank=True)
    feed_title = models.CharField(
        max_length=255, help_text='Title of Feed', null=True, blank=True)
    feed_link = models.URLField(
        max_length=255, help_text='link for Feed', null=True, blank=True)
    feed_description = models.CharField(
        max_length=255, help_text='Description of field', null=True,
        blank=True)
    feed_author_email = models.EmailField(
        max_length=255, help_text='Email of author', null=True, blank=True)
    feed_author_link = models.URLField(
        max_length=255, help_text='Link of author', null=True, blank=True)

    feed_item_description_field = models.CharField(
        max_length=255, help_text='Description field for feed item',
        null=True, blank=True)
    feed_item_content_field = models.CharField(
        max_length=255, help_text='Content Field for feed item',
        null=True, blank=True)
    feed_image_in_content = models.BooleanField(
        help_text='Add feed image to content encoded field',
        default=True)


# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-13 14:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0028_merge'),
    ]

    operations = [
        migrations.CreateModel(
            name='RSSFeedsSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('feed_app_label', models.CharField(blank=True, help_text=b'blog App whose Feed is to be generated', max_length=255, null=True)),
                ('feed_model_name', models.CharField(blank=True, help_text=b'Model to be used for feed generation', max_length=255, null=True)),
                ('feed_title', models.CharField(blank=True, help_text=b'Title of Feed', max_length=255, null=True)),
                ('feed_link', models.URLField(blank=True, help_text=b'link for Feed', max_length=255, null=True)),
                ('feed_description', models.CharField(blank=True, help_text=b'Description of field', max_length=255, null=True)),
                ('feed_author_email', models.EmailField(blank=True, help_text=b'Email of author', max_length=255, null=True)),
                ('feed_author_link', models.URLField(blank=True, help_text=b'Link of author', max_length=255, null=True)),
                ('feed_item_description_field', models.CharField(blank=True, help_text=b'Description field for feed item', max_length=255, null=True)),
                ('feed_item_content_field', models.CharField(blank=True, help_text=b'Content Field for feed item', max_length=255, null=True)),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.conf.urls import include, url

from wagtail.wagtailcore import urls as wagtail_urls

from wagtail_feeds.feeds import BasicFeed, ExtendedFeed


urlpatterns = [
    url(r'^blog/feed/basic$', BasicFeed(), name='basic_feed'),
    url(r'^blog/feed/extended$', ExtendedFeed(), name='extended_feed'),

    # For anything not caught by a more specific rule above, hand over to
    # Wagtail's serving mechanism
    url(r'', include(wagtail_urls))
]
